@@ -3,7 +3,7 @@ package Array::Iterator;
 use strict;
 use warnings;
 
-our $VERSION = '0.07'; # VERSION
+our $VERSION = '0.09'; # VERSION
 
 ### constructor
 
@@ -84,6 +84,8 @@ sub _getItem {
 	return $iteratee->[$index];
 }
 
+sub _get_item { my $self = shift; $self->_getItem(@_) }
+
 # we need to alter this so its an lvalue
 sub _iterated : lvalue {
     (UNIVERSAL::isa((caller)[0], __PACKAGE__))
@@ -102,8 +104,15 @@ sub iterated {
 }
 
 sub has_next {
-	my ($self) = @_;
-	return ($self->{_current_index} < $self->{_length}) ? 1 : 0;
+	my ($self, $n) = @_;
+
+    if(not defined $n) { $n = 1 }
+    elsif(not $n)      { die "has_next(0) doesn't make sense, did you mean current()?" }
+    elsif($n < 0)      { die "has_next() with negative argument doesn't make sense, perhaps you should use a BiDirectional iterator" }
+
+    my $idx = $self->{_current_index} + ($n - 1);
+
+	return ($idx < $self->{_length}) ? 1 : 0;
 }
 
 sub hasNext { my $self = shift; $self->has_next(@_) }
@@ -126,9 +135,16 @@ sub get_next {
 sub getNext { my $self = shift; $self->get_next(@_) }
 
 sub peek {
-	my ($self) = @_;
-    return undef unless (($self->{_current_index}) < $self->{_length});
-	return $self->_getItem($self->{_iteratee}, ($self->{_current_index}));
+	my ($self, $n) = @_;
+
+    if(not defined $n) { $n = 1 }
+    elsif(not $n)      { die "peek(0) doesn't make sense, did you mean get_next()?" }
+    elsif($n < 0)      { die "peek() with negative argument doesn't make sense, perhaps you should use a BiDirectional iterator" }
+
+    my $idx = $self->{_current_index} + ($n - 1);
+
+    return undef unless ($idx < $self->{_length});
+	return $self->_getItem($self->{_iteratee}, $idx);
 }
 
 sub current {
@@ -151,7 +167,9 @@ sub get_length {
 sub getLength { my $self = shift; $self->get_length(@_) }
 
 1;
+#ABSTRACT: A simple class for iterating over Perl arrays
 
+__END__
 
 =pod
 
@@ -161,7 +179,7 @@ Array::Iterator - A simple class for iterating over Perl arrays
 
 =head1 VERSION
 
-version 0.07
+version 0.09
 
 =head1 SYNOPSIS
 
@@ -211,6 +229,8 @@ no attempt is made to enforce this (although I will if I can find an efficient
 means of doing so). This class only intends to provide a clear and simple
 means of generic iteration, nothing more (yet).
 
+=for Pod::Coverage .+
+
 =head1 METHODS
 
 =head2 Public Methods
@@ -231,11 +251,22 @@ of a HASH reference, with the key, __array__:
 
   my $i = Array::Iterator->new({ __array__ => \@array });
 
-=item B<has_next>
+=item B<has_next([$n])>
 
 This methods returns a boolean. True (1) if there are still more elements in
 the iterator, false (0) if there are not.
 
+Takes an optional positive integer (E<gt> 0) that specifies the position you
+want to check. This allows you to check if there an element at arbitrary position.
+Think of it as an ordinal number you want to check:
+
+  $i->has_next(2);  # 2nd next element
+  $i->has_next(10); # 10th next element
+
+Note that C<has_next(1)> is the same as C<has_next()>.
+
+Throws an exception if C<$n> E<lt>= 0.
+
 =item B<next>
 
 This method returns the next item in the iterator, be sure to only call this
@@ -263,16 +294,26 @@ undefined or false values in the iterator. Of course, if this fits your
 data, then there is no problem. Otherwise I would recommend the C<has_next>/C<next>
 idiom instead.
 
-=item B<peek>
+=item B<peek([$n])>
 
 This method can be used to peek ahead at the next item in the iterator. It
 is non-destructuve, meaning it does not advance the internal pointer. If
 this method is called and attempts to reach beyond the bounds of the iterator,
 it will return undef.
 
+Takes an optional positive integer (E<gt> 0) that specifies how far ahead you want to peek:
+
+  $i->peek(2);  # gives you 2nd next element
+  $i->peek(10); # gives you 10th next element
+
+Note that C<peek(1)> is the same as C<peek()>.
+
+Throws an exception if C<$n> E<lt>= 0.
+
 B<NOTE:> Prior to version 0.03 this method would throw an exception if called
 out of bounds. I decided this was not a good practice, as it made it difficult
-to be able to peek ahead effectively.
+to be able to peek ahead effectively. This not the case when calling with an argument
+that is E<lt>= 0 though, as it's clearly a sign of incorrect usage.
 
 =item B<current>
 
@@ -309,7 +350,7 @@ An lvalue-ed subroutine which allows access to the iterator's internal pointer.
 
 This returns the item being iteratated over, in our case an array.
 
-=item B<_getItem ($iteratee, $index)>
+=item B<_get_item ($iteratee, $index)>
 
 This method is used by all other routines to access items with. Given the iteratee
 and an index, it will return the item being stored in the C<$iteratee> at the index
@@ -461,14 +502,9 @@ Steven Haryanto <stevenharyanto@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 by Steven Haryanto.
+This software is copyright (c) 2013 by Steven Haryanto.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-
-__END__
-#ABSTRACT: A simple class for iterating over Perl arrays
-
